"""\
Scan a directory structure to generate a Python file which when executed will
generate a similar directory structure but with folders and file contents
renamed according to various options.

Effectively this module helps you quickly write code to generate directory
structures from a template.
"""

import os
import base64

def gen_template(
    start_path, 
    base_prepend='', 
    replacements=None,
    ignore_extensions=['pyc', 'swp', 'pyo'],
    ignore_files=[],
    treat_as_binary=[],
):
    if not os.path.exists(start_path) or not os.path.isdir(start_path):
        raise Exception('No such directory %r'%start_path)
    if replacements is None:
        replacements = {}
    if start_path.endswith(os.sep):
        start_path = start_path[:-1]
    for k, v in replacements.items():
        for other_key, other_value in replacements.items():
            if k != other_key:
                # print >> sys.stderr, 'Checking %r and %r, %r'%(k, other_key, k in other_key)
                if k in other_key:
                    raise Exception(
                        'The replacement of %r with %r is not guaranteed '
                        'becuase the value is a substring of another value '
                        '%r associated with %r. Please remove one of these '
                        'and correct manually afterwards.' % (
                            v, k, other_key, other_value
                        )
                    )
    all_used = []
    constants_used = []

    ignored_files = []

    def replace(string):
        # Not the best algorithm, but will do for now
        used = []
        find_string = string[:]
        string = string.replace('%', '%%').replace('\\','\\\\')
        for k in replacements.keys():
            if k in ['--XXXXX--', 'os', 'fp', 'open', 'base64']:
                raise Exception('The key name cannot be %r'%k)
            if k in find_string:
                find_string = find_string.replace(k, '--XXXXX--')
                used.append(k)
                if k not in all_used:
                    all_used.append(k)
            # else:
                # print >> sys.stderr, 'Could not find %s'%k 
        for k in used:
            string = string.replace(k.replace('%', '%%').replace('\\','\\\\'), '%('+replacements[k]+')s')
        if used:
            subst_string = '%dict('
            for k in used:
                subst_string += '%s=%s, '%(replacements[k], replacements[k])
            subst_string = subst_string[:-2]
            subst_string += ')'
        else:
            subst_string = '%dict()'
        return subst_string, string

    templates = []
    render_code = []

    start_dirpath = start_path.strip('/').split('/')[-1]
    paths = [start_dirpath]
    for dirpath, dirnames, filenames in os.walk(start_path):
        for dirname in dirnames:
             path = os.path.join(
                 base_prepend+dirpath[len(start_path)-len(start_dirpath):], 
                 dirname
             )
             if not path in paths:
                 paths.append(path)
        for file in filenames:
            if not file.split('.')[-1] in ignore_extensions:
                path = os.path.join(dirpath, file)
                short_path = path[len(start_path)-len(start_dirpath):]
                # print >> sys.stderr, 'Processing %s'%short_path
                constant = short_path.upper().replace(os.sep, '_').replace(\
                   '.', '_').replace('-','_')
                while constant in constants_used:
                    constant = constant+'_'
                constants_used.append(constant)
                fp = open(path, 'rb')
                content = fp.read()
                fp.close()
                
                # These don't change:
                if short_path in ignore_files:
                    ignored_files.append(short_path)
                else:
                    path_subst, path_string = replace(base_prepend+short_path)
                    if short_path in treat_as_binary:
                        templates.append(
                            """\n%s = '''\\\n%s'''"""%(
                                constant,
                                base64.standard_b64encode(content)
                            )
                         )
                        render_code.append(
                            """\
    if os.path.exists('%s'%s):
        raise Exception('File already exists: '+'%s'%s)
    fp = open('%s'%s, 'wb')
    fp.write(base64.standard_b64decode(%s))
    fp.close()"""%(
                                path_string, path_subst,
                                path_string, path_subst,
                                path_string, path_subst,
                                constant
                            )
                        )
                    else:
                        subst_string, content = replace(content)
                        templates.append(
                            """\n%s = '''\\\n%s'''"""%(
                                constant,
                                content
                            )
                        )
                        render_code.append(
                            """\
    if os.path.exists('%s'%s):
        raise Exception('File already exists: '+'%s'%s)
    fp = open('%s'%s, 'wb')
    fp.write(%s%s)
    fp.close()"""%(
                                path_string, path_subst,
                                path_string, path_subst,
                                path_string, path_subst,
                                constant, 
                                subst_string,
                            )
                        )
    print "import os, base64"
    for constant in templates:
        print constant
    print "def render(%s):"%(', '.join([replacements[k] for k in all_used]))
    for path in paths:
        path_subst, path_string = replace(path)
        print '    if os.path.exists("%s"%s):'%(path_string, path_subst)
        print '        raise Exception("Directory already exists: "+"%s"%s)'%(
            path_string, 
            path_subst
        )
        print '    os.mkdir("%s"%s)'%(path_string, path_subst)
    for render in render_code:
        print render

    print '\nif __name__ == "__main__":'
    print '    render('
    for k in all_used:
        print '        %s=%r,'%(replacements[k], k)
    print '    )'
    failed_to_ignore = []
    for file in ignore_files:
        if file not in ignored_files:
            failed_to_ignore.append(file)
    failed_to_use = []
    for item in replacements.keys():
        if item not in all_used:
            failed_to_use.append(item)
    if failed_to_ignore or failed_to_use:
        raise Exception(
            'Could not find these files to ignore: %r. Did not use these '
            'replacements: %r'%(
                failed_to_ignore,
                failed_to_use,
            )
        )


